use thiserror::Error;

#[derive (Error, Debug)]
pub enum ConfigError {
	#[error ("I/O error")]
	Io (#[from] std::io::Error),
	
	#[error ("UTF-8 decoding failed")]
	Utf8 (#[from] std::string::FromUtf8Error),
	
	#[error ("TOML parsing failed")]
	Toml (#[from] toml::de::Error),
	
	#[error ("base64 decoding failed")]
	Base64Decode (#[from] base64::DecodeError),
	
	#[error ("tripcode not 32 bytes after decoding")]
	TripcodeBadLength,
	
	#[error ("unknown config error")]
	Unknown,
}

// I'm not sure how important this is, but it was already in the code

#[derive (Error, Debug)]
pub enum ShuttingDownError {
	#[error ("Relay is shutting down")]
	ShuttingDown,
}

#[derive (Error, Debug)]
pub enum HandleHttpResponseError {
	#[error ("HTTP error")]
	Http (#[from] http::Error),
	
	#[error ("Missing PTTH magic header")]
	MissingPtthMagicHeader,
	
	#[error ("PTTH magic header is not base64")]
	PtthMagicHeaderNotBase64 (base64::DecodeError),
	
	#[error ("PTTH magic header could not be decoded as MessagePack")]
	PtthMagicHeaderNotMsgPack (rmp_serde::decode::Error),
	
	#[error ("Couldn't tell server something")]
	LostServer,
	
	#[error ("Relaying task panicked")]
	RelayingTaskPanicked (#[from] tokio::task::JoinError),
}

#[derive (Error, Debug)]
pub enum RequestError {
	#[error ("HTTP error")]
	Http (#[from] http::Error),
	
	#[error ("MessagePack encode error")]
	MsgPack (#[from] rmp_serde::encode::Error),
	
	#[error ("Handlebars rendering error")]
	Handlebars (#[from] handlebars::RenderError),
	
	#[error ("Error handling HTTP response")]
	HandleHttpResponse (#[from] HandleHttpResponseError),
	
	#[error ("Error is mysterious!")]
	Mysterious,
}

#[derive (Error, Debug)]
pub enum RelayError {
	#[error ("Handlebars template file error")]
	TemplateFile (#[from] handlebars::TemplateFileError),
	
	#[error ("Hyper error")]
	Hyper (#[from] hyper::Error),
}
